﻿//{$WARNINGS OFF}
//{$WARN SYMBOL_PLATFORM OFF}
//{$WARN UNIT_PLATFORM OFF}
//=============================================================================
//  現在のプロセスで使用しているモジュールを列挙
//
//-----------------------------------------------------------------------------
//
//  2020年11月08日 ～
//
//-----------------------------------------------------------------------------
//
//  【動作確認環境】
//
//  Windows 7 U64(SP1) + Delphi XE5(UP2) Pro VCL-32
//
//  Presented by Mr.XRAY
//  http://mrxray.on.coocan.jp/
//=============================================================================
unit Unit1;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes,
  Vcl.Graphics, Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls;

type
  TForm1 = class(TForm)
    Memo1: TMemo;
    procedure FormCreate(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    { Private 宣言 }
  public
    { Public 宣言 }
  end;

  function EnumModules(AMemo: TMemo): Boolean;

var
  Form1: TForm1;

implementation

uses
 Winapi.PsAPI;

{$R *.dfm}

//=============================================================================
//  フォーム生成時の処理
//=============================================================================
procedure TForm1.FormCreate(Sender: TObject);
begin
  Memo1.Lines.Clear;
  Memo1.ScrollBars := ssBoth;
  Memo1.Font.Size  := 11;
  Memo1.Font.Name  := 'Courier New';
end;

//=============================================================================
//  フォーム表示開始時の処理
//=============================================================================
procedure TForm1.FormShow(Sender: TObject);
begin
  Memo1.Lines.BeginUpdate;
  EnumModules(Memo1);
  Memo1.Lines.EndUpdate;
end;

//-----------------------------------------------------------------------------
//  現在のプロセスで使用しているモジュールを列挙
//
//  uses に Winapi.PsAPI が必要
//-----------------------------------------------------------------------------
function EnumModules(AMemo: TMemo): Boolean;
var
  LhProcess     : THandle;
  LSizeNeeded   : Cardinal;
  LhModuleArray : array of HMODULE;
  LhModule      : HMODULE;
  LModulePath   : string;
  LIndex        : Integer;
begin
  Result := False;

  // 現在のプロセスのハンドルを取得
  LhProcess := OpenProcess(PROCESS_QUERY_INFORMATION or PROCESS_VM_READ,
                           False,
                           GetCurrentProcessId);
  if LhProcess = 0 then
    RaiseLastOSError;

  try
    // モジュールハンドルの値格納に必要なメモリのバイト数を取得
    LSizeNeeded := 0;
    EnumProcessModules(LhProcess, nil, 0, LSizeNeeded);
    if LSizeNeeded = 0 then Exit;

    // モジュールハンドルの値を格納する配列の要素数ょ設定
    SetLength(LhModuleArray, LSizeNeeded div SizeOf(HMODULE));

    // モジュールハンドル値を列挙
    if EnumProcessModules(LhProcess,
                          PHMODULE(LhModuleArray),
                          LSizeNeeded,
                          LSizeNeeded) then begin

      // モジュールがある場所 (フルパス) を取得して表示
      for LIndex := 0 to Length(LhModuleArray) - 1 do begin
        LhModule := LhModuleArray[LIndex];

        // 自分自身 (このプログラム EXE ) は除外
        if LhModule <> HInstance then begin
          LModulePath := GetModuleName(LhModule);
          AMemo.Lines.Add(LModulePath);
        end;
      end;

      AMemo.Lines.Add('モジュール数 : ' +  IntToStr(Length(LhModuleArray) - 1));
      if ModuleIsPackage then begin
        AMemo.Lines.Add('実行時パッケージあり');
      end else begin
        AMemo.Lines.Add('実行時パッケージなし');
      end;

      Result := True;
    end;
  finally
    CloseHandle(LhProcess);
  end;
end;

end.
