﻿//{$WARNINGS OFF}
//{$WARN SYMBOL_PLATFORM OFF}
//{$WARN UNIT_PLATFORM OFF}
//=============================================================================
//  実行時における指定パッケージの情報取得
//
//-----------------------------------------------------------------------------
//
//  2020年11月08日 ～
//
//-----------------------------------------------------------------------------
//
//  【動作確認環境】
//
//  Windows 7 U64(SP1) + Delphi XE5(UP2) Pro VCL-32
//
//  Presented by Mr.XRAY
//  http://mrxray.on.coocan.jp/
//=============================================================================
unit Unit1;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes,
  Vcl.Graphics, Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.StdCtrls, Vcl.ComCtrls;

type
  TForm1 = class(TForm)
    Button1: TButton;
    TreeView1: TTreeView;
    Button2: TButton;
    Button3: TButton;
    procedure FormCreate(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
  private
    { Private 宣言 }
  public
    { Public 宣言 }
  end;

  function EachModuleInfo(ATVItems: TTreeNodes;  AhModule: HMODULE;
    Data: Pointer): Boolean;
  procedure PackageInfoProc(const Name: string; NameType: TNameType;
    Flags: Byte; Param: Pointer);

var
  Form1: TForm1;

implementation

{$R *.dfm}

var
  FContNode : TTreeNode;
  FReqNode  : TTreeNode;

//=============================================================================
//  フォーム生成時の処理
//=============================================================================
procedure TForm1.FormCreate(Sender: TObject);
begin
  TreeView1.Items.Clear;
  TreeView1.Font.Size  := 11;
  TreeView1.Font.Name  := 'Courier New';
end;

//=============================================================================
//  筆者がインストールしたパッケージの情報を取得
//=============================================================================
procedure TForm1.Button1Click(Sender: TObject);
var
  LPkgPath : string;
  LhMODULE : HMODULE;
begin
  LPkgPath := 'C:\Users\Public\Documents\RAD Studio\12.0\Bpl\plEnumModule.bpl';

  LhMODULE := LoadPackage(LPkgPath);

  TreeView1.Items.Clear;
  TreeView1.Items.BeginUpdate;
  EachModuleInfo(TreeView1.Items, LhMODULE, nil);
  TreeView1.Items.EndUpdate;

  TreeView1.TopItem.Expand(True);
  TreeView1.TopItem := TreeView1.Items[0];
end;

//=============================================================================
//  自分自身 (HInstance) の情報を取得
//  自分自身の EXE をモジュールとしての情報
//=============================================================================
procedure TForm1.Button2Click(Sender: TObject);
var
  LhMODULE : HMODULE;
begin
  LhMODULE := HInstance;

  TreeView1.Items.Clear;
  TreeView1.Items.BeginUpdate;
  EachModuleInfo(TreeView1.Items, LhMODULE, nil);
  TreeView1.TopItem := nil;
  TreeView1.Items.EndUpdate;

  TreeView1.TopItem.Expand(True);
  TreeView1.TopItem := TreeView1.Items[0];
end;

//=============================================================================
//  dclstd190.bpl (Delphi XE5 標準コンポーネント) パッケージの情報を取得
//=============================================================================
procedure TForm1.Button3Click(Sender: TObject);
var
  LPkgPath : string;
  LhMODULE : HMODULE;
begin
  LPkgPath := 'C:\Users\Public\Documents\RAD Studio\12.0\Bpl\dclusr190.bpl';

  LhMODULE := LoadPackage(LPkgPath);

  TreeView1.Items.Clear;
  TreeView1.Items.BeginUpdate;
  EachModuleInfo(TreeView1.Items, LhMODULE, nil);
  TreeView1.Items.EndUpdate;

  TreeView1.TopItem.Expand(True);
  TreeView1.TopItem := TreeView1.Items[0];
end;

//-----------------------------------------------------------------------------
//  引数で指定したモジュールの情報を取得して TTreeView に表示
//-----------------------------------------------------------------------------
function EachModuleInfo(ATVItems: TTreeNodes; AhModule: HMODULE;
  Data: Pointer): Boolean;
var
  LRootNode   : TTreeNode;
  LModuleName : string;
  LModuleDesc : string;
  LFlags      : Integer;
begin
  LModuleName := GetModuleName(AhModule);
  LRootNode   := ATVItems.Add (nil, LModuleName);

  LModuleDesc := GetPackageDescription (PChar (LModuleName));
  FContNode   := ATVItems.AddChild (LRootNode, 'Contains');
  FReqNode    := ATVItems.AddChild (LRootNode, 'Requires');

  // GetPackageInfo 手続きを実行
  // 第 2 引数は手続きに渡す任意の値 (Pointer 型)
  // 第 3 引数は手続きからの戻り値
  // 第 4 引数はコールバック関数
  GetPackageInfo(AhModule, ATVItems, LFlags, PackageInfoProc);

  // パッケージの説明テキストがない場合
  // IDE 自身が使用するパッケージと見なして除外
  if LModuleDesc <> '' then begin
    ATVItems.AddChild (LRootNode, 'Description: ' + LModuleDesc);
    
    if LFlags and pfDesignOnly = pfDesignOnly then
      ATVItems.AddChild (LRootNode, 'Design Only');

    if LFlags and pfRunOnly = pfRunOnly then
      ATVItems.AddChild (LRootNode, 'Run Only');
  end;
  Result := True;
end;

//-----------------------------------------------------------------------------
//  GetPackageInfo 手続きのコールバック関数
//  TTreeview のノード FContNode, ReqNode (TTreeNode) の階層下に項目を追加
//  FContNode, FReqNode はルーﾄ直下のノードでグローバル変数
//-----------------------------------------------------------------------------
procedure PackageInfoProc(const Name: string; NameType: TNameType;
  Flags: Byte; Param: Pointer);
var
  FlagStr  : string;
  LTVItems : TTreeNodes;
begin
  LTVItems := TTreeNodes(Param);

  FlagStr := ' ';
  if Flags and ufMainUnit <> 0 then
    FlagStr := FlagStr + 'Main Unit ';

  if Flags and ufPackageUnit <> 0 then
    FlagStr := FlagStr + 'Package Unit ';

  if Flags and ufWeakUnit <> 0 then
    FlagStr := FlagStr + 'Weak Unit ';

  if FlagStr <> ' ' then
    FlagStr := ' (' + FlagStr + ')';

    case NameType of
      ntContainsUnit    : LTVItems.AddChild (FContNode, Name + FlagStr);
      ntRequiresPackage : LTVItems.AddChild (FReqNode, Name);
    end;
end;

end.
